from telepot.namedtuple import InlineKeyboardMarkup, InlineKeyboardButton

from model.task import Task
from model.checklist import Checklist
from model.chat import Chat
import resources
import tasksManager, checklistsManager

'''
Retorna un diccionario con los valores proporcionados
@param text: texto del mensaje
@param params: parámetros del mensaje
@param markup: objeto InlineKeyboardMarkup que acompaña al mensaje
@return msg: diccionario genérico con los valores necesarios para responder a una petición/mensaje 
''' 
def message(text, params, markup):
    msg = {
        'text': text,
        'params': params,
        'markup': markup
    }
    return msg
  
'''
Retorna un string con la información de la checklist proporcionada (nombre, tareas y sus estados)
@param cList: objeto checklist del cual se quiere pasar su información a texto
@return msg: string con la información de la checklist
'''     
def printChecklist(cList):
    msg = ''
    msg += '<b>' + str(cList.getName()) + '</b>\n\n'
    for t in tasksManager.findAllByList(cList.getId()):
        if t.isChecked():
            msg += '✅' + t.getName() + '\n'
        else:
            msg += '❌' + t.getName() + '\n'
    return msg

'''
Retorna un InlineKeyboardMarkup con las opciones especiales de creador
@param _id: identificador del chat con el que se está interactuando 
@param cList: objeto checklist del cual se quiere obtener sus opciones de creador
@param keyboard: objeto InlineKeyboardMarkup con el resto de botones disponibles para cualquier usuario sin privilegios
@return InlineKeyboardMarkup: objeto InlineKeyboardMarkup con todos los botones, inclusive especiales creador
'''  
def enableOptions(_id, cList, keyboard):
    auxKeyboard = list(keyboard)
    auxKeyboard.append([InlineKeyboardButton(text=resources.getResource('Share', Chat.Chats[_id].getLanguage()), switch_inline_query= cList.getName()),
                InlineKeyboardButton(text=resources.getResource('Edit', Chat.Chats[_id].getLanguage()), callback_data= 'Edit-'+ str(cList.getId()))])
    return InlineKeyboardMarkup(inline_keyboard=auxKeyboard)

'''
Retorna un InlineKeyboardMarkup con los botones disponibles para un chat concreto
@param _id: identificador del chat con el que se está interactuando 
@param cList: objeto checklist del cual se quiere obtener sus botones
@param page: página de botones en la que se encuentra el usuario actualmente (para checklists de +5 tareas)
@param options: booleano que indica si hay que incluir entre los botones las opciones de creador o no. Por defecto, options = False
@return InlineKeyboardMarkup: objeto InlineKeyboardMarkup con todos los botones
'''  
def checklistKeyboard(_id, cList, page, options = False):
    auxKeyboard = []
    tasks = tasksManager.findAllByList(cList.getId())
    _min = 0
    _max = len(tasks)
    
    #Si se llama desde un inlineQuery no tenemos el chat_id para mandar msg en idioma correcto,
    #creamos objeto chat y cogemos español por defecto 
    if _id not in Chat.Chats:
        Chat.Chats[_id] = Chat(_id)
   
    if len(tasks) > 5:
        if page * 4 >= len(tasks):
            return
        _min = page * 4
        _max = (page + 1) * 4
        
    for task in tasks[_min:_max]:
        auxKeyboard.append([InlineKeyboardButton(text=str(task.getName()), callback_data= 'Check-' + str(cList.getId()) + '-' + str(task.getName()))])
        
    if len(tasks) > 5:
        auxKeyboard.append([InlineKeyboardButton(text=resources.getResource('Previous', Chat.Chats[_id].getLanguage()), callback_data= 'Previous-' + str(cList.getId()) + '-' + str(page - 1)),
                    InlineKeyboardButton(text=resources.getResource('Next', Chat.Chats[_id].getLanguage()), callback_data= 'Next-' + str(cList.getId()) + '-' + str(page + 1))])
                    
    if options:
        return enableOptions(_id, cList, auxKeyboard)
    else:  
        auxKeyboard.append([InlineKeyboardButton(text=resources.getResource('Clone', Chat.Chats[_id].getLanguage()), callback_data= 'Clone-' + str(cList.getId()))])
        return InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    
'''
Retorna un array con los resultados de llamar a message() para cada chat indicado
@param _id: identificador del chat con el que se está interactuando
@param idList: identificador de checklist del cual se quiere obtener el mensaje
@param text: mensaje que acompaña al texto obtenido con prinChecklist() informando de la última accion realizada
@param updateAll: booleano que indica si se van a actualizar todos los mensajes relacionados con la checklist o solo el del _id
@param page: página de botones en la que se encuentra el usuario actualmente (para checklists de +5 tareas). Por defecto, page = 0
@return result: array de diccionarios con los valores necesarios para responder a la petición del usuario 
'''     
def checklistMessage(_id, idList, text, updateAll, page = 0):
    cList = checklistsManager.findById(idList)
    if _id != 0:
        cList.appendMsg((_id, None))
    
    #Se actualizan/editan todos los msg
    if updateAll:
        result = {}
        params = printChecklist(cList)
        for msg_id in cList.getMessages():
            if msg_id[0] == cList.getOwner():
                markup = checklistKeyboard(msg_id[0], cList, page, True)
            else:
                markup = checklistKeyboard(msg_id[0], cList, page)
                
            result[msg_id] = message(text, params, markup)
    
        return result
        
    #Se realiza un único envío de msg
    else:
        if cList.getOwner() == _id:
            markup = checklistKeyboard(_id, cList, page, True)
        else:
            markup = checklistKeyboard(_id, cList, page)
            
        if markup == None:
                return   
            
        return message(text, printChecklist(cList), markup)

'''
Retorna un mensaje informando al usuario de que no entiende su mensaje/petición
@param _id: identificador del chat con el que se está interactuando 
@return message: diccionario con los valores necesarios para responder al mensaje del usuario 
'''          
def notUnderstand(_id):
    return message(resources.getResource('Error_NotUnderstand', Chat.Chats[_id].getLanguage()), None, None)